/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package java.security.cert;

import java.security.GeneralSecurityException;

/**
 * The exception that is thrown when an access to a {@code CertStore} fails.
 */
public class CertStoreException extends GeneralSecurityException {

    private static final long serialVersionUID = 2395296107471573245L;

    /**
     * Creates a new {@code CertStoreException} with the specified message and
     * cause.
     * 
     * @param msg
     *            the detail message for this exception.
     * @param cause
     *            the cause why the access to the certificate store failed.
     */
    public CertStoreException(String msg, Throwable cause) {
        super(msg, cause);
    }

    /**
     * Creates a new {@code CertStoreException} with the specified cause.
     * 
     * @param cause
     *            the cause why the access to the certificate store failed.
     */
    public CertStoreException(Throwable cause) {
        super(cause);
    }

    /**
     * Creates a new {@code CertStoreException} with the specified message.
     * 
     * @param msg
     *            the detail message for this exception.
     */
    public CertStoreException(String msg) {
        super(msg);
    }

    /**
     * Creates a new {@code CertStoreException}.
     */
    public CertStoreException() {
    }
}
